#*****************************************************************************
#
# Copyright (c) 2000 - 2011, Lawrence Livermore National Security, LLC
# Produced at the Lawrence Livermore National Laboratory
# LLNL-CODE-442911
# All rights reserved.
#
# This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
# full copyright notice is contained in the file COPYRIGHT located at the root
# of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
#
# Redistribution  and  use  in  source  and  binary  forms,  with  or  without
# modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of  source code must  retain the above  copyright notice,
#    this list of conditions and the disclaimer below.
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
#    documentation and/or other materials provided with the distribution.
#  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
#    be used to endorse or promote products derived from this software without
#    specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
# ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
# LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
# DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
# SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
# CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
# LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
# OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
#
# Modifications:
#   David M. Camp, Th Jan 14 11:50:00 PST 2010
#   Added new function ADD_TARGET_DEFINITIONS to add defines to targets.
#   This was needed for the plots to define ENGINE only for the engine build.
#
#   Brad Whitlock, Thu May 27 09:30:00 PDT 2010
#   Wrap setting the compiler in an if-exists test.
#
#   Mark C. Miller, Thu Jul 29 17:37:32 PDT 2010
#   Added block for compiler version comments.
#
#   Brad Whitlock, Tue Aug 17 09:17:19 PDT 2010
#   I added NETCDF_CXX_LIB.
#
#   Brad Whitlock, Wed Oct 27 14:12:19 PDT 2010
#   Make sure that TESSELLATION_LIBRARY gets saved.
#
#   Kathleen Bonnell, Thu Dec 2 15:45:44 MST 2010
#   Allow plugin installs on Windows. 
#   Added BOOST_INCLUDE_DIR, TESSELLATION_LIBRARY. 
#   Expand SLIVR_INCLUDE_DIR.  
#   Add defintions for HAVE_LIBGLEW, HAVE_LIBSLIVR (if VISIT_SLIVR).  
#   Only set TUVOK_LIB if VISIT_TUVOK is ON.  
#   Add preprocessor definitions for Windows.
#   Add VISIT_PYTHON_SCRIPTING.
#
#****************************************************************************/

##
## This file gets generated by VisIt's top-level CMakeLists.txt and gets 
## installed along with VisIt so 3rd party plugins can be developed against an 
## installed version of VisIt.
##

# Build shared libraries since we're building plugins.
SET(BUILD_SHARED_LIBS 1)

# Warn if compilers might be incompatible.
IF(EXISTS /usr/bin/gcc AND
   NOT ${CMAKE_C_COMPILER} STREQUAL /usr/bin/gcc)
  MESSAGE(WARNING "C compiler used for VisIt differs from the one used for
                   the current project!  They may incompatible.")
ENDIF(EXISTS /usr/bin/gcc AND
      NOT ${CMAKE_C_COMPILER} STREQUAL /usr/bin/gcc)

IF(EXISTS /usr/bin/g++ AND
   NOT ${CMAKE_CXX_COMPILER} STREQUAL /usr/bin/g++)
  MESSAGE(WARNING "C++ compiler used for VisIt differs from the one used for
                   the current project!  They may incompatible.")
ENDIF(EXISTS /usr/bin/g++ AND
      NOT ${CMAKE_CXX_COMPILER} STREQUAL /usr/bin/g++)


# The comment below is intended to capture version information
# of the compiler in use at the time CMake was invoked to build
# VisIt. If there is no comment in the intervening lines, then
# compiler version information was not successfully obtained.
# -------------------------------------------------------------
# gcc (GCC) 4.4.4 20100726 (Red Hat 4.4.4-13)
# Copyright (C) 2010 Free Software Foundation, Inc.
# This is free software; see the source for copying conditions.  There is NO
# warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# 
# 
# g++ (GCC) 4.4.4 20100726 (Red Hat 4.4.4-13)
# Copyright (C) 2010 Free Software Foundation, Inc.
# This is free software; see the source for copying conditions.  There is NO
# warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# 
# 
# -------------------------------------------------------------

# Commonly used VisIt include directories
SET(VISIT_COMMON_INCLUDES
    ${VISIT_INCLUDE_DIR}/visit/include
    ${VISIT_INCLUDE_DIR}/visit/common/Exceptions/Database
    ${VISIT_INCLUDE_DIR}/visit/common/Exceptions/Pipeline
    ${VISIT_INCLUDE_DIR}/visit/common/Exceptions/Plotter
    ${VISIT_INCLUDE_DIR}/visit/common/comm
    ${VISIT_INCLUDE_DIR}/visit/common/expr
    ${VISIT_INCLUDE_DIR}/visit/common/icons
    ${VISIT_INCLUDE_DIR}/visit/common/misc
    ${VISIT_INCLUDE_DIR}/visit/common/parser
    ${VISIT_INCLUDE_DIR}/visit/common/plugin
    ${VISIT_INCLUDE_DIR}/visit/common/proxybase
    ${VISIT_INCLUDE_DIR}/visit/common/state
    ${VISIT_INCLUDE_DIR}/visit/common/utility
)

# VisIt options
SET(VISIT_PYTHON_SCRIPTING       ON)
SET(VISIT_SERVER_COMPONENTS_ONLY OFF)
SET(VISIT_ENGINE_ONLY            OFF)
SET(VISIT_DBIO_ONLY              OFF)
SET(VISIT_PARALLEL               ON)
SET(VISIT_JAVA                   OFF)
SET(VISIT_SLIVR                  OFF)
SET(VISIT_TUVOK                  OFF)

# Set up VTK
SET(VTK_INCLUDE_DIRS ${VISIT_INCLUDE_DIR}/vtk/include/vtk-5.0)
SET(VTK_LIBRARY_DIRS ${VISIT_LIBRARY_DIR})

# Set up Mesa
SET(MESA_INCLUDE_DIR ${VISIT_INCLUDE_DIR}/mesa/include)
SET(MESA_LIBRARY_DIR ${VISIT_LIBRARY_DIR})

# Set up GLEW
SET(GLEW_INCLUDE_DIR ${VISIT_INCLUDE_DIR}/visit/third_party_builtin/glew/glew/include)
SET(GLEW_LIBRARY_DIR ${VISIT_LIBRARY_DIR})
SET(GLEW_LIB         GLEW)
ADD_DEFINITIONS(-DHAVE_LIBGLEW)

# Set up OpenGL
SET(OPENGL_INCLUDE_DIR  /usr/include)
SET(OPENGL_gl_LIBRARY   /usr/lib64/libGL.so)
SET(OPENGL_glu_LIBRARY  /usr/lib64/libGLU.so)

# Set up the tessellation library
SET(TESSELLATION_LIBRARY /usr/lib64/libGLU.so;/usr/lib64/libGL.so)

# Set up BOOST
SET(BOOST_INCLUDE_DIR ${VISIT_INCLUDE_DIR}/visit/third_party_builtin/bilib/boost_1_42_0)

# Set up Qt
SET(QT_INCLUDE_DIR           ${VISIT_INCLUDE_DIR}/qt/include)
SET(QT_QTCORE_INCLUDE_DIR    ${VISIT_INCLUDE_DIR}/qt/include/QtCore)
SET(QT_QTCORE_LIB            QtCore)

SET(QT_QTGUI_INCLUDE_DIR     ${VISIT_INCLUDE_DIR}/qt/include/QtGui)
SET(QT_QTGUI_LIB             QtGui)

SET(QT_QTNETWORK_INCLUDE_DIR ${VISIT_INCLUDE_DIR}/qt/include/QtNetwork)
SET(QT_QTNETWORK_LIB         QtNetwork)

SET(QT_QTOPENGL_INCLUDE_DIR  ${VISIT_INCLUDE_DIR}/qt/include/QtOpenGL)
SET(QT_QTOPENGL_LIB          QtOpenGL)

SET(QT_QTXML_INCLUDE_DIR     ${VISIT_INCLUDE_DIR}/qt/include/QtXml)
SET(QT_QTXML_LIB             QtXml)

SET(QT_LIBRARY_DIR           ${VISIT_LIBRARY_DIR})
SET(QT_BIN                   ${VISIT_BINARY_DIR})
SET(QT_MOC_EXECUTABLE        ${VISIT_BINARY_DIR}/moc)

# Set up Python
SET(PYTHON_INCLUDE_PATH      ${VISIT_INCLUDE_DIR}/python/include/python2.6)
SET(PYTHON_LIBRARY           ${VISIT_LIBRARY_DIR}/libpython2.6.so)

# Set up SLIVR
SET(SLIVR_INCLUDE_DIR 
    ${VISIT_INCLUDE_DIR}/visit/third_party_builtin/slivr
    ${VISIT_INCLUDE_DIR}/visit/third_party_builtin/slivr/slivr
    ${VISIT_INCLUDE_DIR}/visit/third_party_builtin/slivr/teem-1.9.0-src/src
    ${VISIT_INCLUDE_DIR}/visit/third_party_builtin/slivr/teem-1.9.0-src/src/teem
)
SET(SLIVR_GUI_IMPL    QvisCMap2Display.C QvisCMap2Widget.C)
SET(SLIVR_GUI_HDR     QvisCMap2Display.h QvisCMap2Widget.h)
SET(SLIVR_LIB         slivrG slivrV)

IF(VISIT_SLIVR)
    ADD_DEFINITIONS(-DHAVE_LIBSLIVR)
ENDIF(VISIT_SLIVR)

# Set up 3rd party I/O libraries
SET(ADIOS_INCLUDE_DIR ${VISIT_INCLUDE_DIR}/adios/include)
SET(ADIOS_LIBRARY_DIR ${VISIT_LIBRARY_DIR} ${VISIT_ARCHIVE_DIR})
SET(ADIOS_LIB libadiosread.a)

SET(ADVIO_INCLUDE_DIR ${VISIT_INCLUDE_DIR}/advio/include)
SET(ADVIO_LIBRARY_DIR ${VISIT_LIBRARY_DIR} ${VISIT_ARCHIVE_DIR})
SET(ADVIO_LIB libAdvDocIO.a;libAdvFileIO.a;libAdvBase.a)

SET(BOXLIB2D_INCLUDE_DIR ${VISIT_INCLUDE_DIR}/boxlib2d/2D)
SET(BOXLIB2D_LIBRARY_DIR ${VISIT_LIBRARY_DIR} ${VISIT_ARCHIVE_DIR})
SET(BOXLIB2D_LIB )

SET(BOXLIB3D_INCLUDE_DIR ${VISIT_INCLUDE_DIR}/boxlib3d/3D)
SET(BOXLIB3D_LIBRARY_DIR ${VISIT_LIBRARY_DIR} ${VISIT_ARCHIVE_DIR})
SET(BOXLIB3D_LIB )

SET(CCMIO_INCLUDE_DIR ${VISIT_INCLUDE_DIR}/ccmio/include)
SET(CCMIO_LIBRARY_DIR ${VISIT_LIBRARY_DIR} ${VISIT_ARCHIVE_DIR})
SET(CCMIO_LIB libccmio.a;libadf.a)

SET(CFITSIO_INCLUDE_DIR ${VISIT_INCLUDE_DIR}/cfitsio/include)
SET(CFITSIO_LIBRARY_DIR ${VISIT_LIBRARY_DIR} ${VISIT_ARCHIVE_DIR})
SET(CFITSIO_LIB libcfitsio.a)

SET(CGNS_INCLUDE_DIR ${VISIT_INCLUDE_DIR}/cgns/include)
SET(CGNS_LIBRARY_DIR ${VISIT_LIBRARY_DIR} ${VISIT_ARCHIVE_DIR})
SET(CGNS_LIB libcgns.a;libhdf5.so;libsz.so;libz.so)

SET(EXODUSII_INCLUDE_DIR ${VISIT_INCLUDE_DIR}/exodusii/inc)
SET(EXODUSII_LIBRARY_DIR ${VISIT_LIBRARY_DIR} ${VISIT_ARCHIVE_DIR})
SET(EXODUSII_LIB libexoIIv2c.a;libnetcdf.a;libhdf5_hl.so;libhdf5.so;libsz.so;libz.so)

SET(FASTBIT_INCLUDE_DIR ${VISIT_INCLUDE_DIR}/fastbit/include)
SET(FASTBIT_LIBRARY_DIR ${VISIT_LIBRARY_DIR} ${VISIT_ARCHIVE_DIR})
SET(FASTBIT_LIB libfastbit.a;librt.so)

SET(GDAL_INCLUDE_DIR ${VISIT_INCLUDE_DIR}/gdal/include)
SET(GDAL_LIBRARY_DIR ${VISIT_LIBRARY_DIR} ${VISIT_ARCHIVE_DIR})
SET(GDAL_LIB libgdal.a)

SET(HDF4_INCLUDE_DIR ${VISIT_INCLUDE_DIR}/hdf4/include)
SET(HDF4_LIBRARY_DIR ${VISIT_LIBRARY_DIR} ${VISIT_ARCHIVE_DIR})
SET(HDF4_LIB libmfhdf.a;libdf.a;libsz.so;libvtkjpeg.so)

SET(HDF5_INCLUDE_DIR ${VISIT_INCLUDE_DIR}/hdf5/include)
SET(HDF5_LIBRARY_DIR ${VISIT_LIBRARY_DIR} ${VISIT_ARCHIVE_DIR})
SET(HDF5_LIB libhdf5.so;libsz.so;libz.so)

SET(H5PART_INCLUDE_DIR ${VISIT_INCLUDE_DIR}/h5part/include)
SET(H5PART_LIBRARY_DIR ${VISIT_LIBRARY_DIR} ${VISIT_ARCHIVE_DIR})
SET(H5PART_LIB libH5Part.a;libhdf5.so;libsz.so;libz.so)

SET(MILI_INCLUDE_DIR ${VISIT_INCLUDE_DIR}/mili/include)
SET(MILI_LIBRARY_DIR ${VISIT_LIBRARY_DIR} ${VISIT_ARCHIVE_DIR})
SET(MILI_LIB )

SET(NETCDF_INCLUDE_DIR ${VISIT_INCLUDE_DIR}/netcdf/include)
SET(NETCDF_LIBRARY_DIR ${VISIT_LIBRARY_DIR} ${VISIT_ARCHIVE_DIR})
SET(NETCDF_LIB libnetcdf.a;libhdf5_hl.so;libhdf5.so;libsz.so;libz.so)
SET(NETCDF_CXX_LIB libnetcdf_c++.a)

SET(PDB_INCLUDE_DIR ${VISIT_INCLUDE_DIR}/silo/include)
SET(PDB_LIBRARY_DIR ${VISIT_LIBRARY_DIR} ${VISIT_ARCHIVE_DIR})
SET(PDB_LIB libsiloh5.a;libhdf5.so;libsz.so;libz.so)

SET(SILO_INCLUDE_DIR ${VISIT_INCLUDE_DIR}/silo/include)
SET(SILO_LIBRARY_DIR ${VISIT_LIBRARY_DIR} ${VISIT_ARCHIVE_DIR})
SET(SILO_LIB libsiloh5.a;libhdf5.so;libsz.so;libz.so)

SET(VISUS_INCLUDE_DIR  ${VISIT_INCLUDE_DIR}/visus/include)
SET(VISUS_LIBRARY_DIR  ${VISIT_LIBRARY_DIR} ${VISIT_ARCHIVE_DIR})
SET(VISUS_LIB )

SET(XDMF_INCLUDE_DIR  ${VISIT_INCLUDE_DIR}/xdmf/include)
SET(XDMF_LIBRARY_DIR  ${VISIT_LIBRARY_DIR} ${VISIT_ARCHIVE_DIR})
SET(XDMF_LIB libXdmf.so;libvtklibxml2.so;libhdf5.so;libsz.so;libz.so)

# Set up TUVOK
IF(VISIT_TUVOK)
    SET(TUVOK_LIB         tuvok)
ENDIF(VISIT_TUVOK)

# Windows specific defines
IF(WIN32)
  ADD_DEFINITIONS(-D_USE_MATH_DEFINES)
  ADD_DEFINITIONS(-D_CRT_SECURE_NO_DEPRECATE -D_CRT_NONSTDC_NO_DEPRECATE)
  ADD_DEFINITIONS(-D_SCL_NO_DEPRECATE -D_SCL_SECURE_NO_DEPRECATE)
  ADD_DEFINITIONS(-D_CRT_SECURE_NO_WARNINGS)
ENDIF(WIN32)

# Installation macros
MACRO(VISIT_INSTALL_DATABASE_PLUGINS)
    FOREACH(target ${ARGN})
        SET_TARGET_PROPERTIES(${target} PROPERTIES
           RUNTIME_OUTPUT_DIRECTORY "${VISIT_PLUGIN_DIR}/databases"
           LIBRARY_OUTPUT_DIRECTORY "${VISIT_PLUGIN_DIR}/databases"
        )
    ENDFOREACH(target)
ENDMACRO(VISIT_INSTALL_DATABASE_PLUGINS)

MACRO(VISIT_INSTALL_OPERATOR_PLUGINS)
    FOREACH(target ${ARGN})
        SET_TARGET_PROPERTIES(${target} PROPERTIES
           RUNTIME_OUTPUT_DIRECTORY "${VISIT_PLUGIN_DIR}/operators"
           LIBRARY_OUTPUT_DIRECTORY "${VISIT_PLUGIN_DIR}/operators"
        )
    ENDFOREACH(target)
ENDMACRO(VISIT_INSTALL_OPERATOR_PLUGINS)

MACRO(VISIT_INSTALL_PLOT_PLUGINS)
    FOREACH(target ${ARGN})
        SET_TARGET_PROPERTIES(${target} PROPERTIES
           RUNTIME_OUTPUT_DIRECTORY "${VISIT_PLUGIN_DIR}/plots"
           LIBRARY_OUTPUT_DIRECTORY "${VISIT_PLUGIN_DIR}/plots"
        )
    ENDFOREACH(target)
ENDMACRO(VISIT_INSTALL_PLOT_PLUGINS)

# Parallel settings
SET(VISIT_PARALLEL_CXXFLAGS     -DPARALLEL -DMPICH_IGNORE_CXX_SEEK  -I/usr/local/openmpi/1.4.3/gcc/x86_64/include -I/usr/local/include )
SET(VISIT_PARALLEL_LINKER_FLAGS  -Wl,--export-dynamic)
SET(VISIT_PARALLEL_LIBS         /usr/lib64/libpvfs2.so;/usr/lib64/libpthread.so;/usr/local/openmpi/1.4.3/gcc/x86_64/lib64/libmpi_cxx.so;/usr/local/openmpi/1.4.3/gcc/x86_64/lib64/libmpi.so;/usr/local/openmpi/1.4.3/gcc/x86_64/lib64/libopen-rte.so;/usr/local/openmpi/1.4.3/gcc/x86_64/lib64/libopen-pal.so;/usr/lib64/libdl.so;/usr/lib64/libpvfs2.so;/usr/lib64/libpthread.so;/usr/lib64/libnsl.so;/usr/lib64/libutil.so;/usr/lib64/libm.so)

FUNCTION(ADD_PARALLEL_LIBRARY target)
    ADD_LIBRARY(${target} ${ARGN})
    IF(VISIT_PARALLEL_CXXFLAGS)
        SET(PAR_COMPILE_FLAGS "")
        FOREACH(X ${VISIT_PARALLEL_CXXFLAGS})
            SET(PAR_COMPILE_FLAGS "${PAR_COMPILE_FLAGS} ${X}")
        ENDFOREACH(X)
        SET_TARGET_PROPERTIES(${target} PROPERTIES
            COMPILE_FLAGS ${PAR_COMPILE_FLAGS}
        )
        IF(VISIT_PARALLEL_LINKER_FLAGS)
            SET(PAR_LINK_FLAGS "")
            FOREACH(X ${VISIT_PARALLEL_LINKER_FLAGS})
                SET(PAR_LINK_FLAGS "${PAR_LINK_FLAGS} ${X}")
            ENDFOREACH(X)
            SET_TARGET_PROPERTIES(${target} PROPERTIES
                LINK_FLAGS ${PAR_LINK_FLAGS}
            )
        ENDIF(VISIT_PARALLEL_LINKER_FLAGS)
        TARGET_LINK_LIBRARIES(${target} ${VISIT_PARALLEL_LIBS})
    ENDIF(VISIT_PARALLEL_CXXFLAGS)
ENDFUNCTION(ADD_PARALLEL_LIBRARY)

FUNCTION(ADD_TARGET_DEFINITIONS target newDefs)
    GET_TARGET_PROPERTY(currentDefs ${target} COMPILE_DEFINITIONS)
    IF(${currentDefs} MATCHES NOTFOUND)
        SET(allDefs ${newDefs})
    ELSE(${currentDefs} MATCHES NOTFOUND)
        SET(allDefs ${currentDefs} ${newDefs})
    ENDIF(${currentDefs} MATCHES NOTFOUND)
    SET_TARGET_PROPERTIES(${target} PROPERTIES
        COMPILE_DEFINITIONS ${allDefs}
    )
ENDFUNCTION(ADD_TARGET_DEFINITIONS)

MACRO(VISIT_PLUGIN_TARGET_PREFIX) 
    IF(WIN32)
        SET_TARGET_PROPERTIES(${ARGN} PROPERTIES PREFIX "../lib")
    ENDIF(WIN32)
ENDMACRO(VISIT_PLUGIN_TARGET_PREFIX)
