/*****************************************************************************
*
* Copyright (c) 2000 - 2011, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

// ************************************************************************* //
//                        avtSamplePointToSurfaceFilter.h                    //
// ************************************************************************* //

#ifndef AVT_SAMPLE_POINT_TO_SURFACE_FILTER_H
#define AVT_SAMPLE_POINT_TO_SURFACE_FILTER_H

#include <filters_exports.h>

#include <avtSamplePointsToDatasetFilter.h>

class     avtImagePartition;


typedef enum
{
    FRONT_SURFACE =      0,
    MIDDLE_SURFACE,   /* 1 */
    BACK_SURFACE,     /* 2 */
    NOT_SPECIFIED     /* 3 */
} SurfaceType;


// ****************************************************************************
//  Class: avtSamplePointToSurfaceFilter
//
//  Purpose:
//      A filter that will take in sample points and create a surface.  The
//      surface can be the surface at the front of the samples, the back of
//      the samples, or in the middle of the samples.
//
//  Programmer: Hank Childs
//  Creation:   June 30, 2003
//
//  Modifications:
//
//    Hank Childs, Thu Feb  5 17:11:06 PST 2004
//    Moved inlined destructor definition to .C file because certain compilers
//    have problems with them.
//
// ****************************************************************************

class AVTFILTERS_API avtSamplePointToSurfaceFilter 
    : public avtSamplePointsToDatasetFilter
{
  public:
                              avtSamplePointToSurfaceFilter();
    virtual                  ~avtSamplePointToSurfaceFilter();

    void                      SetSurfaceType(SurfaceType st)
                                    { surfType = st; };
    void                      SetImagePartition(avtImagePartition *ip)
                                    { imagePartition = ip; };
    
    virtual const char       *GetType(void)
                                    { return "avtSamplePointToSurfaceFilter"; };
    virtual const char       *GetDescription(void)
                                    { return "Creating surface from samples";};

  protected:
    SurfaceType               surfType;
    avtImagePartition        *imagePartition;

    virtual void              Execute(void);
};


#endif


