/*****************************************************************************
*
* Copyright (c) 2000 - 2011, Lawrence Livermore National Security, LLC
* Produced at the Lawrence Livermore National Laboratory
* LLNL-CODE-442911
* All rights reserved.
*
* This file is  part of VisIt. For  details, see https://visit.llnl.gov/.  The
* full copyright notice is contained in the file COPYRIGHT located at the root
* of the VisIt distribution or at http://www.llnl.gov/visit/copyright.html.
*
* Redistribution  and  use  in  source  and  binary  forms,  with  or  without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of  source code must  retain the above  copyright notice,
*    this list of conditions and the disclaimer below.
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this  list of  conditions  and  the  disclaimer (as noted below)  in  the
*    documentation and/or other materials provided with the distribution.
*  - Neither the name of  the LLNS/LLNL nor the names of  its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT  HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR  IMPLIED WARRANTIES, INCLUDING,  BUT NOT  LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS FOR A PARTICULAR  PURPOSE
* ARE  DISCLAIMED. IN  NO EVENT  SHALL LAWRENCE  LIVERMORE NATIONAL  SECURITY,
* LLC, THE  U.S.  DEPARTMENT OF  ENERGY  OR  CONTRIBUTORS BE  LIABLE  FOR  ANY
* DIRECT,  INDIRECT,   INCIDENTAL,   SPECIAL,   EXEMPLARY,  OR   CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT  LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR
* SERVICES; LOSS OF  USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER
* CAUSED  AND  ON  ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT
* LIABILITY, OR TORT  (INCLUDING NEGLIGENCE OR OTHERWISE)  ARISING IN ANY  WAY
* OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
*
*****************************************************************************/

#ifndef EXPRESSION_H
#define EXPRESSION_H
#include <state_exports.h>
#include <string>
#include <AttributeSubject.h>


// ****************************************************************************
// Class: Expression
//
// Purpose:
//    This class contains an expression.
//
// Notes:      Autogenerated by xml2atts.
//
// Programmer: xml2atts
// Creation:   omitted
//
// Modifications:
//   
// ****************************************************************************

class STATE_API Expression : public AttributeSubject
{
public:
    enum ExprType
    {
        Unknown,
        ScalarMeshVar,
        VectorMeshVar,
        TensorMeshVar,
        SymmetricTensorMeshVar,
        ArrayMeshVar,
        CurveMeshVar,
        Mesh,
        Material,
        Species
    };

    // These constructors are for objects of this class
    Expression();
    Expression(const Expression &obj);
protected:
    // These constructors are for objects derived from this class
    Expression(private_tmfs_t tmfs);
    Expression(const Expression &obj, private_tmfs_t tmfs);
public:
    virtual ~Expression();

    virtual Expression& operator = (const Expression &obj);
    virtual bool operator == (const Expression &obj) const;
    virtual bool operator != (const Expression &obj) const;
private:
    void Init();
    void Copy(const Expression &obj);
public:

    virtual const std::string TypeName() const;
    virtual bool CopyAttributes(const AttributeGroup *);
    virtual AttributeSubject *CreateCompatible(const std::string &) const;
    virtual AttributeSubject *NewInstance(bool) const;

    // Property selection methods
    virtual void SelectAll();
    void SelectName();
    void SelectDefinition();
    void SelectOperatorName();
    void SelectMeshName();
    void SelectDbName();

    // Property setting methods
    void SetName(const std::string &name_);
    void SetDefinition(const std::string &definition_);
    void SetHidden(bool hidden_);
    void SetType(ExprType type_);
    void SetFromDB(bool fromDB_);
    void SetFromOperator(bool fromOperator_);
    void SetOperatorName(const std::string &operatorName_);
    void SetMeshName(const std::string &meshName_);
    void SetDbName(const std::string &dbName_);
    void SetAutoExpression(bool autoExpression_);

    // Property getting methods
    const std::string &GetName() const;
          std::string &GetName();
    const std::string &GetDefinition() const;
          std::string &GetDefinition();
    bool              GetHidden() const;
    ExprType          GetType() const;
    bool              GetFromDB() const;
    bool              GetFromOperator() const;
    const std::string &GetOperatorName() const;
          std::string &GetOperatorName();
    const std::string &GetMeshName() const;
          std::string &GetMeshName();
    const std::string &GetDbName() const;
          std::string &GetDbName();
    bool              GetAutoExpression() const;

    // Persistence methods
    virtual bool CreateNode(DataNode *node, bool completeSave, bool forceAdd);
    virtual void SetFromNode(DataNode *node);

    // Enum conversion functions
    static std::string ExprType_ToString(ExprType);
    static bool ExprType_FromString(const std::string &, ExprType &);
protected:
    static std::string ExprType_ToString(int);
public:

    // Keyframing methods
    virtual std::string               GetFieldName(int index) const;
    virtual AttributeGroup::FieldType GetFieldType(int index) const;
    virtual std::string               GetFieldTypeName(int index) const;
    virtual bool                      FieldsEqual(int index, const AttributeGroup *rhs) const;

    // User-defined methods
    static int GetNumTypes();
    static const char * GetTypeString(const Expression::ExprType t);
    static Expression::ExprType GetTypeId(const std::string s);
    

    // IDs that can be used to identify fields in case statements
    enum {
        ID_name = 0,
        ID_definition,
        ID_hidden,
        ID_type,
        ID_fromDB,
        ID_fromOperator,
        ID_operatorName,
        ID_meshName,
        ID_dbName,
        ID_autoExpression,
        ID__LAST
    };

private:
    std::string name;
    std::string definition;
    bool        hidden;
    int         type;
    bool        fromDB;
    bool        fromOperator;
    std::string operatorName;
    std::string meshName;
    std::string dbName;
    bool        autoExpression;

    // Static class format string for type map.
    static const char *TypeMapFormatString;
    static const private_tmfs_t TmfsStruct;
};
#define EXPRESSION_TMFS "ssbibbsssb"

#endif
